from abc import ABCMeta, abstractmethod

from externals.util import BaseThread


class DeviceServiceThread(BaseThread):
    __metaclass__ = ABCMeta

    @abstractmethod
    def run(self):
        # type: () -> None

        raise NotImplementedError()
