
class Command(object):

    def __init__(self, raw_data):
        # type: (dict) -> None

        self.id = raw_data["id"]
        self.type = raw_data["type"]
        self.data = None

        if self.type == "ReprocessSales":
            self.data = raw_data["data"]["fromDate"]
