from abc import ABCMeta, abstractmethod
from typing import Optional

from nperipherals_core.model.device import Device


class ReaderDevice(Device):
    __metaclass__ = ABCMeta

    @abstractmethod
    def read(self):
        # type: () -> Optional[str]
        raise NotImplementedError()
