class NavigationProduct(object):
    def __init__(self, nav_id, class_code, product_code, name, background_color):
        self.nav_id = nav_id
        self.class_code = class_code
        self.product_code = product_code
        self.name = name
        self.background_color = background_color
