from ._NetworkDevice import NetworkDevice
from ._NetworkScannerDevice import NetworkScannerDevice
from ._NetworkEscPosPrinterDevice import NetworkEscPosPrinterDevice

try:
    from ._WindowsApiPrinterDevice import WindowsApiPrinterDevice
except ImportError:
    class WindowsApiPrinterDevice(object):
        pass
