# -*- coding: utf-8 -*-

# noinspection SpellCheckingInspection
class NfeCardBrand(object):
    # Tabela de bandeiras de Cartões de Crédito/Débito - Publicada em 16/03/2021
    # http://www.nfe.fazenda.gov.br/portal/listaConteudo.aspx?tipoConteudo=Iy/5Qol1YbE=
    # http://www.nfe.fazenda.gov.br/portal/exibirArquivo.aspx?conteudo=txtcX/WzVFY=

    visa = 1
    mastercard = 2
    american_express = 3
    sorocred = 4
    diners_club = 5
    elo = 6
    hipercard = 7
    aura = 8
    cabal = 9
    alelo = 10
    banes_card = 11
    cal_card = 12
    credz = 13
    discover = 14
    good_card = 15
    green_card = 16
    hiper = 17
    jc_b = 18
    mais = 19
    max_van = 20
    policard = 21
    rede_compras = 22
    sodexo = 23
    vale_card = 24
    verocheque = 25
    vr = 26
    ticket = 27
    outros = 99


