from helper import ExtendedEnum
from messagebus import TokenPriority, TokenCreator

MESSAGE_GROUP = "96"


class ListenedMessages(ExtendedEnum):
    TK_CUSTOMER_ADD_CUSTOMER = TokenCreator.create_token(TokenPriority.low, MESSAGE_GROUP, "1")
    TK_CUSTOMER_GET_CUSTOMER = TokenCreator.create_token(TokenPriority.low, MESSAGE_GROUP, "2")
    TK_CUSTOMER_UPDATE_CUSTOMER = TokenCreator.create_token(TokenPriority.low, MESSAGE_GROUP, "3")
    TK_CUSTOMER_DELETE_CUSTOMER = TokenCreator.create_token(TokenPriority.low, MESSAGE_GROUP, "4")
