from typing import Dict

from ._Route import Route


class FoundRoute(object):
    def __init__(self, route, path_parameters):
        # type: (Route, Dict[str, str]) -> None
        self.route = route
        self.path_parameters = path_parameters
